//Programme de Test 3D pour l'ecran TFT2.2" de www.TechToys.com.hk
// Controlleur SSD1289 de Solomon
// Example repris du source Microchip PIC18 C Compiler
//
// Utiliser le driver SSD1289.c
// A Changer dans SSD1289.c si les broches utilis sont differentes
// 
// #define  glcd_wr             PIN_C0
// #define  glcd_rd             PIN_C1
// #define  glcd_cs             PIN_C2
// #define  glcd_cd             PIN_C6
//
// set_tris_C et D de la fonction LCD_Init si necessaire
//
// Le port de donnes de l'ecran D1  D8 est connect au PORT D D0 -> D7
//
// F. Muller 20 Juin 2007
// CCS C version : 4.032
// Pour PIC18F4680 32Mhz (Int_Osc a 8Mhz ,PLL x 4)

#include <SSD1289_18F4680_3D.h>
#include <stdlib.h>
#include <string.h>
#include <SSD1289.c>
#include <trigotable.h>

void Obj3D (void);                   // Calculer et tracer le cube


// Variables globales composant les points et les faces du Cube
const signed int px[8]={-10,10,10,-10,-10,10,10,-10};   // Les points X du cube
const signed int py[8]={-10,-10,10,10,-10,-10,10,10};   // Les points Y du cube
const signed int pz[8]={-10,-10,-10,-10,10,10,10,10};   // Les points Z du cube

const int face[12][3]={{0,1,2},{2,3,0},{1,5,6},{6,2,1},    // Les faces du cube
                       {5,4,7},{7,6,5},{4,0,3},{3,7,4},
                       {3,2,6},{6,7,3},{0,4,5},{5,1,0}};


void main()
{
   
   setup_adc_ports(AN0_TO_AN1|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL|ADC_TAD_MUL_0);
   setup_psp(PSP_DISABLED);
   setup_spi(FALSE);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_oscillator(OSC_32MHZ);

   // Stabilisation de l'alimentation
   delay_ms(1000);
   
   set_tris_b(0x00);
   output_b (0);

   // Toujour initialiser le LCD en 1er
   LCD_Init();

   // Effacer l'ecran en NOIR
   LCD_CLS(NOIR);

   // Lancer la routine d'affichage du Cube 3D
   Obj3D();
}

void Obj3D()
  {
   // Calculer et tracer le cube
   float  tx,ty,tz,x,y,z;              // Coordonnes temporaires
   int i;                              // Compteur
   float sx,sy,sz,cx,cy,cz;            // Sin et Cos precalcul
   INT16U ax,ay,az;                    // Les angles de 0  359 degres
   INT8U dx,zoom;                      // Centrage x,y et zoom du cube
   INT16U dy;
   INT8U  nx[8];                       // Coordonnes de tracage
   INT16U ny[8];
   INT8U oldx[8];                      // Sauvegarde des ancienne coordonnes
   INT16U oldy[8];


   // Angles de dpart
   ax = 45;
   ay = 20;
   az = 0;

   // Zoom du cube
   zoom = 3;

   // Centrage du cube
   dx = 119;
   dy = 159;

   // Rotation du cube
   While(1)
   {

    // Tranfers des sin et cos prcalcul dans les variables de travail
    sx = si[ax]; sy = si[ay]; sz = si[az];
    cx = co[ax]; cy = co[ay]; cz = co[az];


    // Calculer les nouvelles coordonnes des points
    for (i=0; i<8; i++)
        {
           tx = px[i]; ty = py[i]; tz = pz[i];

           // Sauvegarde des anciennes coordonnes
           oldx[i] = nx[i]; oldy[i] = ny[i];

           // Rotation sur Axe X
           y = ty * cx - tz * sx;
           z = ty * sx + tz * cx;
           ty =y; tz = z;
           // Rotation en Z
           x = tx * cz - ty * sz;
           y = tx * sz + ty * cz;
           tx = x; ty = y;
           // Rotation en Y
           x = tx * cy - tz * sy;
           z = tx * sy + tz * cy;
           // Sauver les coordonnes ecran
           nx[i] = (INT8U)x * zoom + dx;
           ny[i] = (INT16U)y * zoom + dy;
        }
        delay_ms(5);

    // Effacer l'ancien cube et afficher le nouveau
    for (i=0; i<12; i++)
        { 
             LCD_Ligne(oldx[face[i][0]],oldy[face[i][0]],oldx[face[i][1]],oldy[face[i][1]], NOIR);
             LCD_Ligne(oldx[face[i][1]],oldy[face[i][1]],oldx[face[i][2]],oldy[face[i][2]], NOIR);
             //LCD_Ligne(oldx[face[i][2]],oldy[face[i][2]],oldx[face[i][0]],oldy[face[i][0]], WHITE);
             LCD_Ligne(nx[face[i][0]],ny[face[i][0]],nx[face[i][1]],ny[face[i][1]], BLEU);
             LCD_Ligne(nx[face[i][1]],ny[face[i][1]],nx[face[i][2]],ny[face[i][2]], BLEU);
             //LCD_Ligne(nx[face[i][2]],ny[face[i][2]],nx[face[i][0]],ny[face[i][0]], BLACK);
        }
    // Incrementer les angles
    ax = ax + 3; if (ax > 359) ax = ax - 360;
    ay = ay + 3; if (ay > 359) ay = ay - 360;
    az = az + 3; if (az > 359) az = az - 360;
   }
  }

